@@ -1,4 +1,4 @@
-/*** analog 2.1 ***/
+/*** analog 3.0 ***/
 /* Please read Readme.html, or http://www.statslab.cam.ac.uk/~sret1/analog/  */
 
 /*** macstuff.c; stuff only required for the Mac port ***/
@@ -6,6 +6,8 @@
    copyright, Jason Linhart (jason@summary.net), 1996. */
 /* The functions in this file inside #ifdef MACDIRENT are due to
    Jason Linhart, January 1997. */
+/* The functions in this file inside #ifdef MAC are due to Jason Linhart,
+   March 1998. */
 /* The functions in this file inside #ifndef NODNS are due to, and
    copyright, Jason Linhart and Stephan Somogyi, 1996, 1997.
    Version history:
@@ -16,36 +18,71 @@
                     OpenNetwork
 */
 
+#define REAL_ALLOC
 #include "analhea2.h"
 #ifdef MAC_EVENTS
 
-void MacInit(void)
+static char *cmd_vect[64] = {
+	"analog",
+	"+g%%%",
+	0
+	};
+
+extern char mac_do_script[256];
+extern char TryAEOpen(void);
+
+static int
+CmdParse(char **vect,char *cmd_line,int limit)
 {
+        int count;
+
+        for (count=0; count<limit && *cmd_line; ++count) {
+                if (*cmd_line=='"') {
+                        ++cmd_line;
+                        *vect++ = cmd_line;
+                        while (*cmd_line && *cmd_line!='"') ++cmd_line;
+                        if (*cmd_line) *cmd_line++ = 0;
+                        }
+                else {
+                        *vect++ = cmd_line;
+                        while (*cmd_line && *cmd_line!=' ' && *cmd_line!='\t') ++cmd_line;
+                        if (*cmd_line) *cmd_line++ = 0;
+                        }
+                while (*cmd_line==' ' || *cmd_line=='\t') ++cmd_line;
+                }
+        return(count);
+        }
+
+
+void MacInit(int *argc,char **argv[])
+{
+  char c;
+
   SIOUXSettings.asktosaveonclose=false;
-  printf("Processing...\n");
+  (void)printf("Processing...\n");
+  if ((c=TryAEOpen())!=0) {
+  	*argc=2;
+  	if (c!='#') cmd_vect[1]=cmd_vect[1]+2;
+  	}
+  else if (mac_do_script[0]) 
+        *argc=1+CmdParse(cmd_vect+1,mac_do_script,63);
+  else *argc=1;
+  *argv=cmd_vect;
 }
 
+void mac_temp_free(void);
+
 void MacFini(void)
 {
-  extern char *commandname;
-  extern long Mac_good_lines, Mac_bad_lines, Mac_tab_lines;  /* in sscanf.c */
-  extern int total_succ_reqs;
-  extern flag warnq, anywarns;
+  extern logical anywarns;
 
-  if (!Mac_good_lines && Mac_tab_lines > 4 && total_succ_reqs < 5 && warnq) {
-    fprintf(stderr,"%s: Warning: No valid log entries found!\n", commandname);
-    fprintf(stderr,"WebStar logs must start with a LOG_FORMAT line something like:\n");
-    fprintf(stderr,"!!LOG_FORMAT DATE TIME RESULT URL TRANSFER_TIME BYTES_SENT HOSTNAME REFERER\n");
-    fprintf(stderr,"except with the fields listed in the order they actually occur.\n");
-    fprintf(stderr,"WebSTAR 1.2.1 or higher should create this line automatically.\n\n");
-    anywarns = ON;
-  }
   if (!anywarns)
     SIOUXSettings.autocloseonquit=true;
-  printf("Complete!\n");
 #ifndef NODNS
   ResolverCleanup();
 #endif
+  (void)fprintf(stderr, "Complete!\n");
+  mac_temp_free();
 }
 
 void MacIdle(void)
@@ -57,7 +94,6 @@ void MacIdle(void)
   char theChar;
 
   if (TickCount()<time) return;
-  time=TickCount()+8;
   SystemTask();
   if (WaitNextEvent(everyEvent, &myEvent, 1, nil)) {
 
@@ -89,6 +125,7 @@ void MacIdle(void)
       break;
     }
   }
+  time=TickCount()+20;
 }
 #endif   /* (ifdef MAC_EVENTS) */
 
@@ -100,10 +137,8 @@ static CInfoPBRec finfo;
 static unsigned char fname[257];
 
 static void
-CToPCpy(pstr,cstr)                 /* Convert a C string to a Pascal string */
-        unsigned char *pstr;
-        char *cstr;
-{
+CToPCpy(unsigned char *pstr, const char *cstr)
+{                 /* Convert a C string to a Pascal string */
         register char *dptr, len;
 
         len=0;
@@ -238,7 +273,8 @@ dirstat(const char *file_name, struct stat *buf)
 /* Special version of stat that only works for most recent dir entry */
 {
         if (indx) {
-                buf->st_mode=finfo.hFileInfo.ioFlAttrib;
+                if (finfo.hFileInfo.ioFlAttrib&0x10) buf->st_mode=S_IFDIR;
+                else buf->st_mode=S_IFREG;
                 return(0);
                 }
         return(-1);
@@ -327,7 +363,7 @@ int IpAddr2Name(char *hostname)
     }
     if (err == noErr) {
       hInfoMacTCP.cname[254] = 0;
-      strcpy(hostname, hInfoMacTCP.cname);
+      (void)strcpy(hostname, hInfoMacTCP.cname);
       if (hostname[strlen(hostname)-1]=='.') hostname[strlen(hostname)-1]=0;
       return(1);
     }
@@ -429,3 +465,169 @@ OpenNetwork(void)
 } /* end OpenNetwork() */
 #endif
 #endif
+
+#ifdef MAC
+
+#define MY_MAGIC_ONE		0x3748596A
+#define MY_MAGIC_TWO		0xA7958473
+
+#define MY_BIG_SIZE			(32*1024)
+#define MY_BLOCK_SIZE		(4*(MY_BIG_SIZE+3*sizeof(long)))
+
+#if MY_BIG_SIZE!=BLOCKSIZE
+Look into this! They should be the same, but changes would
+require other adjustments.
+#endif
+
+static long my_cur_avail = 0;
+static char *my_cur_block;
+static struct my_block {
+	struct my_block **next;
+	char storage[MY_BLOCK_SIZE];
+	} **temp_mem_blocks = nil;
+
+static struct my_free {
+	struct my_free *next;
+	char storage[MY_BIG_SIZE-sizeof(struct my_free *)];
+	} *my_free_list = nil;
+
+void mac_temp_free(void);
+
+void
+mac_temp_free(void)
+{
+	OSErr result;
+	struct my_block **block, **next;
+
+	block=temp_mem_blocks;
+	while (block) {
+		next=(*block)->next; 
+		TempDisposeHandle((Handle)block,&result);
+		if (result) break;
+		block=next;
+		}
+	}
+
+static void *
+my_temp_malloc(size_t size)
+{
+	OSErr result;
+	struct my_block **block;
+	long version;
+	char *ret_ptr;
+
+	size=(size+3)&~3;
+	if (Gestalt(gestaltSystemVersion,&version) || version<0x700) return(nil);
+	if (size>MY_BLOCK_SIZE) {
+		if (TempFreeMem()<1024*1024+size) return(nil);
+		block=(struct my_block **)TempNewHandle(size+sizeof(struct my_block **),&result);
+		if (!block || !*block || result) return(nil);
+		TempHLock((Handle)block,&result);
+		(*block)->next=temp_mem_blocks;
+		temp_mem_blocks=block;
+		return((void *)&(*block)->storage);
+		}
+	if (size>my_cur_avail) {
+		if (TempFreeMem()<1024*1024+MY_BLOCK_SIZE) return(nil);
+		block=(struct my_block **)TempNewHandle(sizeof(struct my_block),&result);
+		if (!block || !*block || result) return(nil);
+		TempHLock((Handle)block,&result);
+		(*block)->next=temp_mem_blocks;
+		temp_mem_blocks=block;
+		my_cur_block=(*block)->storage;
+		my_cur_avail=MY_BLOCK_SIZE;
+		}
+	ret_ptr=my_cur_block;
+	my_cur_block+=size;
+	my_cur_avail-=size;
+	return((void *)ret_ptr);
+	}
+
+static void *
+sub_mac_malloc(size_t size)
+{
+	char *ret_ptr;
+
+	size=(size+3)&~3;
+	if (size>=MY_BIG_SIZE) {
+		if (MaxBlock()<16*1024+size) return(my_temp_malloc(size));
+		return(malloc(size));
+		}
+	if (size>my_cur_avail) {
+		if (MaxBlock()<16*1024+MY_BLOCK_SIZE) return(my_temp_malloc(size));
+		my_cur_block=malloc(MY_BLOCK_SIZE);
+		my_cur_avail=MY_BLOCK_SIZE;
+		}
+	ret_ptr=my_cur_block;
+	my_cur_block+=size;
+	my_cur_avail-=size;
+	return((void *)ret_ptr);
+	}
+
+void *
+mac_malloc(size_t size)
+{
+	long *ptr;
+
+	if (size>=MY_BIG_SIZE) {
+		if (size==MY_BIG_SIZE && my_free_list) {
+			ptr=(long *)my_free_list;
+			my_free_list=my_free_list->next;
+			}
+		else {
+			size+=(size/MY_BIG_SIZE)*3*sizeof(long);
+			ptr=(long *)sub_mac_malloc(size);
+			if (ptr) {
+				*ptr++ = size;
+				*ptr++ = MY_MAGIC_ONE;
+				*ptr++ = MY_MAGIC_TWO;
+				}
+			}
+		}
+	else ptr=(long *)sub_mac_malloc(size);
+	return((void *)ptr);
+	}
+
+void *
+mac_calloc(size_t num,size_t size)
+{
+	return(mac_malloc(num*size));
+	}
+
+void *
+mac_realloc(void *ptr,size_t size)
+{
+	void *new_blk;
+
+	if (!size || !(new_blk=mac_malloc(size))) return(nil);
+	memcpy(new_blk,ptr,size);
+	mac_free(ptr);
+	return(new_blk);
+	}
+
+void
+mac_free(void *ptr)
+{
+	struct my_free *blk;
+	long *lptr, size;
+
+	lptr=(long *)ptr;
+	if (lptr) {
+		lptr-=3;
+		if (lptr[1]==MY_MAGIC_ONE && lptr[2]==MY_MAGIC_TWO) {
+			size = *lptr + 3*sizeof(long);
+			while (size>=MY_BIG_SIZE+3*sizeof(long)) {
+				blk=(struct my_free *)(lptr+3);
+				blk->next=my_free_list;
+				my_free_list=blk;
+				*lptr=MY_BIG_SIZE+3*sizeof(long);
+				lptr[1]=MY_MAGIC_ONE;
+				lptr[2]=MY_MAGIC_TWO;
+				lptr+=(MY_BIG_SIZE/sizeof(long))+3;
+				size-=MY_BIG_SIZE+3*sizeof(long);
+				}
+			}
+		}
+	}
+#endif
+
